/*
 * SPC user-space interface.
 */

#ifndef SPC_H
#define SPC_H

#ifdef __KERNEL__
#include <linux/ioctl.h>
#include <linux/time.h>
#else
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/mman.h>
#endif

#define SPC_IOCTL_MAGIC	0x9d

#if !defined(DEFINE_PCI_DEVICE_TABLE)
#define DEFINE_PCI_DEVICE_TABLE(name)\
	static struct pci_device_id name[]
#endif

struct spc_caps {
	char name[16];
	unsigned int core_nr;
	unsigned int flags;
	unsigned long shmem_offs;
	unsigned long shmem_size;
#define SPC_CAP_EXPORT_PMR	(1 << 0)
#define SPC_CAP_DMA		(1 << 1)
#define SPC_CAP_XOR		(1 << 29)
	unsigned int pmr_nr;
#define SPC_CAP_MAX_PMRS	8
	unsigned long pmr_offs[SPC_CAP_MAX_PMRS];
	unsigned long pmr_sizes[SPC_CAP_MAX_PMRS];
};

#define SPC_IOCTL_GET_CAPS \
	_IOR(SPC_IOCTL_MAGIC, 0, struct spc_caps)

#define SPC_SHMEM_BUF_LEN (4096 - sizeof(unsigned long) -\
		sizeof(unsigned long) * 2)

struct spc_buf {
	unsigned long flags;
	unsigned long offset;
	unsigned long size;
	unsigned char data[SPC_SHMEM_BUF_LEN];
} __attribute__((packed));

typedef struct spc_buf spc_buf_aligned __attribute__((aligned(4096)));

#define SPC_SHMEM_BUF_NR 16
struct spc_shmem {
	spc_buf_aligned bufs[SPC_SHMEM_BUF_NR];
};

#define SPC_IOCTL_GET_REV \
	_IOWR(SPC_IOCTL_MAGIC, 20, int)

struct spc_hid {
	char hid[128];
};

#define SPC_IOCTL_GET_HID \
	_IOWR(SPC_IOCTL_MAGIC, 21, struct spc_hid)


#define SPC_IOCTL_CHECK_RESET \
	_IOWR(SPC_IOCTL_MAGIC, 30, int)

#endif /* ifndef SPC_H */
