/*
 * SPC in-kernel data structures and interface.
 */

#ifndef SPC_DEVICE_H
#define SPC_DEVICE_H

#include "spc.h"

#include <linux/device.h>
#include <linux/cdev.h>
#include <linux/hrtimer.h>
#include <linux/mutex.h>
#include <asm/div64.h>

struct spc_ops;
struct spc_fd;

struct spc_device {
	struct spc_caps *caps;
	struct spc_ops *ops;
	struct module *owner;
	struct device dev;
	struct pci_dev *pcid;
	struct cdev cdev;
	int id;
	spinlock_t lock;
	struct spc_fd *fd;
	struct spc_shmem *mem;
	dma_addr_t pmem;
	int rev;
	struct spc_hid hid;
};

static inline struct spc_device *to_spc_device(
		struct device *dev)
{
	return container_of(dev, struct spc_device, dev);
}

extern int spc_register(struct spc_device *dev);
extern void spc_unregister(struct spc_device *dev);

struct spc_ops {
	void (*release)(struct spc_device *dev);
	int (*open)(struct spc_device *dev);
	void (*close)(struct spc_device *dev);
	int (*ready)(struct spc_device *dev, struct spc_buf *buf);
	int (*write)(struct spc_device *sdev, void *vaddr,
		unsigned long paddr, unsigned long size);
	int (*read)(struct spc_device *sdev, void *vaddr,
		unsigned long paddr, unsigned long size);
	int (*mmap)(struct spc_device *dev, struct vm_area_struct *vma);
	void (*munmap)(struct spc_device *dev, struct spc_fd *fd);
	int (*check_reset)(struct spc_device *dev, unsigned int __user *);
	long (*ioctl)(struct spc_fd *fd, struct spc_device *sdev,
		unsigned int cmd, void __user *arg);
};

struct spc_fd {
	struct spc_device *dev;

	int event_mmaped;
	volatile void __iomem *event_mmap_addr;

	int device_mmaped;
	volatile void __iomem *device_mmap_addr;
};

#endif /* ifnnef SPC_DEVICE_H */
